// ----------------------------------------------------------
// This Source Code Form is subject to the terms of the
// Mozilla Public License, v.2.0. If a copy of the MPL
// was not distributed with this file, You can obtain one
// at http://mozilla.org/MPL/2.0/.
// ----------------------------------------------------------
// Codebase: https://github.com/ArKuznetsov/cpdb/
// ----------------------------------------------------------

#Использовать "../../core"

Перем Лог;       // - Объект      - объект записи лога приложения

#Область СлужебныйПрограммныйИнтерфейс

// Процедура - устанавливает описание команды
//
// Параметры:
//  Команда    - КомандаПриложения     - объект описание команды
//
Процедура ОписаниеКоманды(Команда) Экспорт
	
	Команда.Опция("pp params", "", "Файлы JSON содержащие значения параметров,
	                               | могут быть указаны несколько файлов разделенные "";""")
	       .ТСтрока()
	       .ВОкружении("CPDB_PARAMS");

	Команда.Опция("f file", "", "путь к локальному файлу для помещения на Yandex-Диск")
	       .ТСтрока()
	       .ВОкружении("CPDB_YD_PUT_FILE");
	
	Команда.Опция("l list", "", "путь к локальному файлу со списком файлов,
	                            |которые будут помещены на Yandex-Диск
	                            |(параметр -file игнорируется)")
	       .ТСтрока()
	       .ВОкружении("CPDB_YD_PUT_LIST");
	
	Команда.Опция("p yp path ya-path", "", "путь к файлу на Yandex-Диск")
	       .ТСтрока()
	       .ВОкружении("CPDB_YD_PUT_PATH");
	
	Команда.Опция("r replace", "", "перезаписать файл на Яндекс-диске при загрузке")
	       .Флаговый()
		   .ПоУмолчанию(Ложь)
	       .ВОкружении("CPDB_YD_PUT_REPLACE");
	
	Команда.Опция("ds delsrc", "", "удалить исходные файлы после отправки")
	       .Флаговый()
		   .ПоУмолчанию(Ложь)
	       .ВОкружении("CPDB_YD_PUT_DEL_SRC");
	
КонецПроцедуры // ОписаниеКоманды()

// Процедура - запускает выполнение команды устанавливает описание команды
//
// Параметры:
//  Команда    - КомандаПриложения     - объект  описание команды
//
Процедура ВыполнитьКоманду(Знач Команда) Экспорт

	ЧтениеОпций = Новый ЧтениеОпцийКоманды(Команда);

	ВыводОтладочнойИнформации = ЧтениеОпций.ЗначениеОпции("verbose");

	ПараметрыСистемы.УстановитьРежимОтладки(ВыводОтладочнойИнформации);

	ЭтоСписокФайлов = Истина;

	ПутьКФайлу              = ЧтениеОпций.ЗначениеОпции("list");
	Если НЕ ЗначениеЗаполнено(ПутьКФайлу) Тогда
		ПутьКФайлу          = ЧтениеОпций.ЗначениеОпции("file");
		ЭтоСписокФайлов	= Ложь;
	КонецЕсли;
	OAuth_Токен             = ЧтениеОпций.ЗначениеОпции("token");
	ЦелевойПуть             = ЧтениеОпций.ЗначениеОпции("path");
	УдалитьИсточник         = ЧтениеОпций.ЗначениеОпции("delsrc");
	Перезаписывать          = ЧтениеОпций.ЗначениеОпции("replace");
		
	Если ПустаяСтрока(ПутьКФайлу) Тогда
		ВызватьИсключение "Не указан путь к файлу для помещения на Yandex-Диск";
	КонецЕсли;
	
	МассивОтправляемыхФайлов = Новый Массив;
	
	РаботаСЯндексДиск = Новый РаботаСЯндексДиск(OAuth_Токен);

	// Если целевой путь не указан - тогда используется корень Яндекс-диска
	Если ЗначениеЗаполнено(ЦелевойПуть) Тогда
		// Определяем наличие каталога
		РаботаСЯндексДиск.СоздатьКаталог(ЦелевойПуть);
	Иначе
		ЦелевойПуть = "";
	КонецЕсли;
	
	Если ЭтоСписокФайлов Тогда
		МассивОтправляемыхФайлов = РаботаСФайлами.ПрочитатьСписокФайлов(ПутьКФайлу, Истина);
	Иначе
		ФайлИнфо = Новый Файл(ПутьКФайлу);
		МассивОтправляемыхФайлов.Добавить(ФайлИнфо.ПолноеИмя);
	КонецЕсли;

	Для Каждого ОтправляемыйФайл Из МассивОтправляемыхФайлов Цикл
		РаботаСЯндексДиск.ОтправитьФайл(ОтправляемыйФайл, ЦелевойПуть, Перезаписывать);

		Если УдалитьИсточник Тогда
			УдалитьФайлы(ОтправляемыйФайл);
			Лог.Информация("Исходный файл %1 удален", ОтправляемыйФайл);
		КонецЕсли;
	
	КонецЦикла;

КонецПроцедуры // ВыполнитьКоманду()

#КонецОбласти // СлужебныйПрограммныйИнтерфейс

#Область ОбработчикиСобытий

// Процедура - обработчик события "ПриСозданииОбъекта"
//
// BSLLS:UnusedLocalMethod-off
Процедура ПриСозданииОбъекта()

	Лог = ПараметрыСистемы.Лог();

КонецПроцедуры // ПриСозданииОбъекта()
// BSLLS:UnusedLocalMethod-on

#КонецОбласти // ОбработчикиСобытий
